@@ -11,6 +11,8 @@ import os.path
 import HErmes as he
 import hepbasestack as hep
 
+import hjson
+
 from .utils import get_stripname
 logger = hep.logger.get_logger(10)
 
@@ -54,6 +56,29 @@ CONSTANTS = Constants()
 
 ########################################################################
 
+def serialize_noisemodel(noisemodel,\
+                         filename):
+    """
+    Save the noisemodel to a file, serialize via json
+
+    Args:
+        noisemodel (HErmes.fitting.model.Model) : the fitted noisemodel
+        filename   (str)                        : full path to file
+    """
+    nm_data = dict()
+    nm_data['I_L']      = noisemodel.I_L
+    nm_data['A_f']      = noisemodel.A_f
+    nm_data['R_S']      = noisemodel.R_S
+    nm_data['chi2/ndf'] = noisemodel.chi2_ndf
+    nm_data['npoint']   = len(noisemodel.data)
+    nm_data['detid']    = noisemodel.detid     
+    nm_data['strip']    = noisemodel.stripname 
+    logger.info(f'Serializing noisemodel to {filename}')
+    hjson.dump(nm_data, open(filename,'w'))
+    return None
+
+########################################################################
+
 def extract_parameters_from_noisemodel(noisemodel,\
                                        method='trapezoid'):
     """
@@ -271,7 +296,7 @@ def construct_error_belt(nm, xs=None):
 def fit_noisemodel(xs, ys, ys_err,  ch, detid,\
                    plotdir='.', fig=None,\
                    use_minuit=True,\
-                   use_refined_model=False,\
+                   use_direct_model=False,\
                    debug_minuit=False,
                    method='trapezoid'):
     """
@@ -279,28 +304,33 @@ def fit_noisemodel(xs, ys, ys_err,  ch, detid,\
     with the noisemodel
 
     Args:
-        xs (ndarray)         : peaking times (microseconds)
-        ys (ndarray)         : x-ray resolutions (fwhm)
-        ys_err (ndarray)     : fwhm associated fit errors
-        ch (int)             : digitizer channel/detector strip
-        detid (int)          : detector id (needed for the filename to save the plot)
+        xs           (ndarray)   : peaking times (microseconds)
+        ys           (ndarray)   : x-ray resolutions (fwhm)
+        ys_err       (ndarray)   : fwhm associated fit errors
+        ch               (int)   : digitizer channel/detector strip
+        detid            (int)   : detector id (needed for the filename to save the plot)
 
     Keyword Args:
         fig (Figure)             : use a pre-exisiting figure instance. If None, create new
         plotdir (str)            : directory to save the plot in
-        use_minuit (bool)        : Use minuit for the minimization (recommended)
-        use_refined_model (bool) : use a slightly different implempentation of the noise
-                                   model, which should yield the exact same numerical values
-        debug_minuit (bool)      : pass this parameter to the model. It attaches the
+        use_minuit       (bool)  : Use minuit for the minimization (recommended)
+        use_direct_model (bool)  : use a slightly different implempentation of the noise
+                                   model. The difference is that the 3 physics parameters
+                                   I_L, R_S and A_f are fitted directly.
+        debug_minuit     (bool)  : pass this parameter to the model. It attaches the
                                    iminuit instance to the model, so it is accessible for
                                    later debugging
+        fig (matplotlib.Figure)  : A predefined figure install to use
+        method            (str)  : Tell the noisemodel what method was used to obtain the 
+                                   data, either a trapezoidal shaper or a gaussian. 
+                                   This affects the weights for the physics parameters. 
     Returns:
-        None
+        tuple (matplotlib.figure, model)               
     """
     # channel noise model fit
     logger.info(f'Performing noise model fit for channel {ch}')
-    if use_refined_model:
-        logger.info(f'Use refined noise model...')
+    if use_direct_model:
+        logger.info(f'Using direct model, fitting I_L, R_S, A_f...')
         noisemodel = he.fitting.Model(preamp_noise_model)
         noisemodel.startparams = (2e-9,1, 8e-13)
     else:
@@ -310,7 +340,7 @@ def fit_noisemodel(xs, ys, ys_err,  ch, detid,\
     noisemodel.add_data(ys, xs=xs/1000,\
                         data_errs=ys_err,
                         create_distribution=False)
-    if use_refined_model:
+    if use_direct_model:
         noisemodel.fit_to_data(use_minuit=use_minuit, \
                                debug_minuit=debug_minuit, \
                                errors=(1, 1, 1), \
@@ -332,7 +362,7 @@ def fit_noisemodel(xs, ys, ys_err,  ch, detid,\
                                    (0,100)))
 
     logger.info(f'Noisemodel fitted, best fit pars: {noisemodel.best_fit_params}')
-    if use_refined_model:
+    if use_direct_model:
         if isinstance(noisemodel.errors, list):
             tmpdict = dict()
             tmpdict['par00'] = noisemodel.errors[0]
@@ -368,9 +398,9 @@ def fit_noisemodel(xs, ys, ys_err,  ch, detid,\
     else:
         params = extract_parameters_from_noisemodel(noisemodel, method=method)
         logger.info(f'Extracted {params} from noisemodel')
-        print(f'..I_L {params["Ileak"]}')
-        print(f'..A_f {params["Af"]}')
-        print(f'..R_s {params["Rs"]}')
+        logger.debug(f'..I_L {params["Ileak"]}')
+        logger.debug(f'..A_f {params["Af"]}')
+        logger.debug(f'..R_s {params["Rs"]}')
         I_L = params["Ileak"]
         A_f = params["Af"]
         R_S = params["Rs"]
@@ -413,7 +443,7 @@ def fit_noisemodel(xs, ys, ys_err,  ch, detid,\
     ax.set_yscale('log')
     ax.set_ylim(bottom=1)
     ax.grid(which='minor', color='gray', alpha=0.7)
-    title = f'det{detid}-{get_stripname(ch)}-nmfit'
+    title = f'det{detid}-{get_stripname(ch)}'
     ax.set_title(title, loc='right')
     ax.set_xlabel('peaking time [$\mu$s]')
     ax.set_ylabel('xray res. (FWHM) [keV]')
@@ -424,15 +454,18 @@ def fit_noisemodel(xs, ys, ys_err,  ch, detid,\
             size='xx-small', \
             bbox=dict(facecolor='white', alpha=0.7, edgecolor=None), \
             )
-    pngfilename = os.path.join(plotdir, title + '.png')
+    pngfilename = os.path.join(plotdir, title + '-nmfit.png')
+    noisemodel_file = os.path.join(plotdir, title + '-nmfit.dat') 
     noisemodel_fig.savefig(pngfilename)
-    logger.info(f'Saved file {pngfilename}')
+    logger.warn(f'Saved file {pngfilename}')
     # attach some metainformation
     noisemodel.I_L = I_L
     noisemodel.A_f = A_f
     noisemodel.R_S = R_S
     noisemodel.detid     = detid
     noisemodel.stripname = get_stripname(ch)
+    serialize_noisemodel(noisemodel,\
+                         noisemodel_file)
     return noisemodel_fig, noisemodel
 
 ########################################################################
